local _, class = UnitClass("player")
if class ~= "DEATHKNIGHT" then
	return
end


mRunes = LibStub("AceAddon-3.0"):NewAddon("mRunes", "AceEvent-3.0", "AceConsole-3.0")
local mRunes = mRunes

-- Locals
local db
local OptionsFrame
local mRuneFrame
local runes = {}
local runicPowerBar

-- Rune Types
local RUNETYPE_BLOOD = 1;
local RUNETYPE_UNHOLY = 2;
local RUNETYPE_FROST = 3;
local RUNETYPE_DEATH = 4;

-- GUI
local UIF
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")

function mRunes:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			-- Frame values
			FrameFadeInRate = 1,
			FrameFadeOutRate = 0.5,
			FrameInCombatAlpha = 1,
			FrameOutOfCombatAlpha = 0.1,
			FrameExtend = 2,
			FrameBackgroundColor = {0, 0, 0, 0.5},
			PositionX = 0,
			PositionY = 0,
			-- Rune values
			RuneHeight = 20,
			RuneWidth = 60,
			RuneSpacing = 1,
			RuneTexture = "Blizzard",
			RuneColors = {
				[RUNETYPE_BLOOD] = {1.0, 0.0, 0.0, 1},
				[RUNETYPE_UNHOLY] ={0.0, 0.8, 0.0, 1},
				[RUNETYPE_FROST] = {0.0, 1.0, 1.0, 1},
				[RUNETYPE_DEATH] = {0.8, 0.1, 1.0, 1},
			},
			-- Runic Power values
			RunicPowerColor = {0.00, 0.82, 1.00},
			RunicPowerHeight = 15,
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mRunesDB", defaults, "Default")
	db = self.db.profile
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self.runes = runes
end

function mRunes:OnEnable()
	-- Hide Blizzard frame
	RuneFrame:Hide()
	
	if not mRuneFrame then
		self:CreateRuneFrame()
		self:CreateRunes()
		self:ApplyAllSettings()
	else
		mRuneFrame:Show()
	end
	
	if UnitAffectingCombat("player") then self:EnterCombat() else self:LeaveCombat() end
	
	-- Register Events
	self:RegisterEvent("RUNE_POWER_UPDATE", "RuneStateChange")
	self:RegisterEvent("RUNE_TYPE_UPDATE", "RuneTypeChange")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnterWorld") 
	self:RegisterEvent("UNIT_MAXRUNIC_POWER", "RunicPowerChange")
	self:RegisterEvent("UNIT_RUNIC_POWER", "RunicPowerChange")
	
	self:RegisterChatCommand("mr", "ToggleConfig")
	
	local UIFSettings = {
		s = {
			font = LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"),
			fontSize = 16,
		}
	}

	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mRunes", UIFSettings)
	UIF:SetValuesChangedCallback(function() mRunes:ApplyAllSettings() end)
end

function mRunes:OnDisable()
	-- Show Blizzard frame
	RuneFrame:Show()
	mRuneFrame:Hide()
end


---------------------------------------------------
-- Events
---------------------------------------------------
do
	function mRunes:RuneStateChange(event, rune, state)
		if (rune == 7 or rune == 8) then
			return
		end
		
		local _, _, Ready = GetRuneCooldown(rune)
		if not Ready then
			runes[rune]:SetScript("OnUpdate", mRunes.UpdateRune)
		end
	end

	function mRunes:RuneTypeChange(event, rune)
		self:UpdateRuneType(runes[rune])			
	end
	
	function mRunes:EnterCombat()
		mRuneFrame.fadeType = "IN"
		mRuneFrame.fadeOutTime = nil
		mRuneFrame:SetScript("OnUpdate", self.FrameUpdate)
	end
	
	function mRunes:LeaveCombat()
		mRuneFrame.fadeType = "OUT"
		mRuneFrame:SetScript("OnUpdate", self.FrameUpdate)
	end
	
	function mRunes:EnterWorld()
		self:UpdateAll()
	end
end

---------------------------------------------------
-- Creation
---------------------------------------------------
do
	function mRunes:CreateRuneFrame()
		local frame = CreateFrame("Frame", "mRunesRuneFrame", UIParent)
		frame.bg = frame:CreateTexture()
		frame.bg:SetAllPoints()
		frame:SetAlpha(0)
		mRuneFrame = frame
	end

	function mRunes:CreateRunes()
		for i = 1, 6 do
			local rune = self:CreateRune(i)			
			runes[i] = rune
		end
		runicPowerBar = CreateFrame("StatusBar", "mRunesRunicPowerBar", mRuneFrame)
		runicPowerBar:SetPoint("TOPLEFT", runes[1], "BOTTOMLEFT", 0, -db.FrameExtend * 1)
		runicPowerBar:SetPoint("TOPRIGHT", runes[6], "BOTTOMRIGHT", 0, -db.FrameExtend * 1)		
		runicPowerBar:SetHeight(15)
		runicPowerBar:SetStatusBarTexture(LSM:Fetch("statusbar", db.RuneTexture))
		runicPowerBar:SetStatusBarColor(unpack(db.RunicPowerColor))
		runicPowerBar:SetMinMaxValues(0, 1)
		runicPowerBar:SetValue(UnitPower("player") / UnitPowerMax("player"))
		
		local runicText = runicPowerBar:CreateFontString()
		runicText:SetPoint("RIGHT", -3, 0)
		runicText:Hide()
		runicText:SetFont(LSM:Fetch("font", "accid"), 16, "OUTLINE")
		
		runicPowerBar.text = runicText
	end

	function mRunes:CreateRune(runeID)
		local rune = CreateFrame("StatusBar", "mRunesRune"..runeID, mRuneFrame)
		rune:SetMinMaxValues(0,1)	
		
		local runeCooldown = rune:CreateFontString()
		runeCooldown:SetFont(LSM:Fetch("font", "accid"), 16, "OUTLINE")
		runeCooldown:SetPoint("TOPLEFT", 2, 0)
		runeCooldown:SetPoint("BOTTOMRIGHT", 0, 0)
		runeCooldown:SetJustifyH("LEFT")
		runeCooldown:Hide()
		rune.CDText = runeCooldown
		
		rune.ID = runeID
		
		self:ApplySettings(rune)
		rune:SetScript("OnUpdate", mRunes.UpdateRune)
		
		
		return rune
	end
end

---------------------------------------------------
-- Updates
---------------------------------------------------
do
	function mRunes:FrameUpdate(elapsed)
		self = mRunes
		local fadeType = mRuneFrame.fadeType
		
		if fadeType then
			local currAlpha = mRuneFrame:GetAlpha()
			if fadeType == "IN" then
				if currAlpha > db.FrameInCombatAlpha then
					mRuneFrame:SetAlpha(db.FrameInCombatAlpha)
					mRuneFrame.fadeType = nil
				else
					mRuneFrame:SetAlpha(currAlpha + db.FrameFadeInRate * elapsed)
				end
			else
				if currAlpha < db.FrameOutOfCombatAlpha then
					mRuneFrame:SetAlpha(db.FrameOutOfCombatAlpha)
					mRuneFrame.fadeType = nil
				else
					mRuneFrame:SetAlpha(currAlpha - db.FrameFadeOutRate * elapsed)
				end
			end
		else
			mRuneFrame:SetScript("OnUpdate", nil)
		end
		
	end

	function mRunes:UpdateRune(elapsed)
		local rune = self
		local self = self or mRunes
				
		local StartTime, CooldownTime, Ready = GetRuneCooldown(rune.ID)
		if not Ready then
			local diff = GetTime() - StartTime
			local remaining = CooldownTime - diff
			local t = diff / CooldownTime
			rune.CDText:Show()			
			rune:SetValue(t)
			
			if remaining < 0 then rune.CDText:Hide() else rune.CDText:SetText(("%.1f"):format(remaining)) end
		else
			rune:SetValue(1)
			rune.CDText:Hide()
			rune:SetScript("OnUpdate", nil)
		end
	end
	
	function mRunes:RunicPowerChange(event, ...)
		local power = UnitPower("player")
		local maxPower = UnitPowerMax("player")
		if power > 0 then
			runicPowerBar:SetValue(power / maxPower)
			runicPowerBar.text:Show()
			runicPowerBar.text:SetText(power.." / "..maxPower)
		else
			runicPowerBar:SetValue(0)
			runicPowerBar.text:SetText("0 / "..maxPower)
			runicPowerBar.text:Hide()
		end
	end
	
	function mRunes:UpdateRuneType(rune)
		rune:SetStatusBarColor(unpack(db.RuneColors[GetRuneType(rune.ID)]))
	end
	
	function mRunes:UpdateAll()
		for i, rune in pairs(runes) do
			rune:SetScript("OnUpdate", mRunes.UpdateRune)
			mRunes:UpdateRuneType(rune)
		end
	end
end

---------------------------------------------------
-- Settings
---------------------------------------------------
do
	function mRunes:ApplyAllSettings()
		self = mRunes
		for i, rune in pairs(runes) do
			mRunes:ApplySettings(rune)
		end
		
		mRuneFrame:SetPoint("CENTER", db.PositionX, db.PositionY)
		mRuneFrame.bg:SetTexture(unpack(db.FrameBackgroundColor))
		mRuneFrame:SetWidth(6 * (db.RuneWidth + db.RuneSpacing) + 2 * db.FrameExtend)
		mRuneFrame:SetHeight(db.RuneHeight + (3 * db.FrameExtend) + db.RunicPowerHeight)
		
		if UnitAffectingCombat("player") then 
			self:EnterCombat() 
		else
			self:LeaveCombat() 
		end
	end

	function mRunes:ApplySettings(rune)
		rune:SetWidth(db.RuneWidth)
		rune:SetHeight(db.RuneHeight)
		if rune.ID == 1 then
			rune:SetPoint("TOPLEFT", mRuneFrame, "TOPLEFT", db.FrameExtend,  -db.FrameExtend)
		else
			rune:SetPoint("LEFT", runes[rune.ID - 1], "RIGHT", db.RuneSpacing, 0)
		end
		rune:SetStatusBarTexture(LSM:Fetch("statusbar", db.RuneTexture))
		rune:SetStatusBarColor(unpack(db.RuneColors[GetRuneType(rune.ID)]))	
	end
end

---------------------------------------------------
-- Config
---------------------------------------------------
do	
	local function ChangeCallback(widget, event, ...)
	end

	function mRunes:ToggleConfig()
		if not OptionsFrame then	
			local m = UIF:Frame("mRunes")
			m:SetLayout("Fill")
			m:SetWidth(810)
			m:SetHeight(620)
			m:SetCallback("OnClose", function(widget)
				widget:ReleaseChildren()
				widget:Release()
				OptionsFrame = nil
			end)				
			m:Show()			
			OptionsFrame = m
			
			m = AceGUI:Create("ScrollFrame")
			m:SetLayout("Flow")
			OptionsFrame:AddChild(m)
			
			local select, slider, color, group, group2, check
			
			check = UIF:CheckBox("Enabled", db, "Enabled", function()
				if db.Enabled then
					mRunes:Enable()
				else
					mRunes:Disable()
				end
			end)
			m:AddChild(check)
			
			group = UIF:InlineGroup("Frame")
			slider = UIF:Slider("In combat Alpha", db, "FrameInCombatAlpha", 0, 1, 0.01, mRunes.ApplyAllSettings)
			group:AddChild(slider)
			slider = UIF:Slider("Out of combat Alpha", db, "FrameOutOfCombatAlpha", 0, 1, 0.01, mRunes.ApplyAllSettings)
			group:AddChild(slider)
			group2 = UIF:InlineGroup2("Position")
			slider = UIF:Slider("X", db, "PositionX", 
				-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
				floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
				0.01, mRunes.ApplyAllSettings)
			group2:AddChild(slider)
			slider = UIF:Slider("Y", db, "PositionY", 
				-floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
				floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
				0.01, mRunes.ApplyAllSettings)
			group2:AddChild(slider)
			group:AddChild(group2)
			m:AddChild(group)
			
			group = UIF:InlineGroup("Background")
			color = UIF:ColorSelect("Color", db, "FrameBackgroundColor",  mRunes.ApplyAllSettings)
			color:SetHasAlpha(true)
			group:AddChild(color)
			slider = UIF:Slider("Extend", db, "FrameExtend", 0, 50, 0.1, mRunes.ApplyAllSettings)
			group:AddChild(slider)
			m:AddChild(group)
			
			group = UIF:InlineGroup("Runes")
			m:AddChild(group)
			
			select = UIF:LSMDropdown("Statusbar", "Rune Texture", db, "RuneTexture",  mRunes.ApplyAllSettings)
			group:AddChild(select)
			
			group2 = UIF:InlineGroup2("Colors")
			color = UIF:ColorSelect("Blood Rune", db.RuneColors, RUNETYPE_BLOOD,  mRunes.ApplyAllSettings)
			color:SetRelativeWidth(0.20)
			color:SetHasAlpha(true)
			group2:AddChild(color)
			color = UIF:ColorSelect("Unholy Rune", db.RuneColors, RUNETYPE_UNHOLY,  mRunes.ApplyAllSettings)
			color:SetRelativeWidth(0.20)
			color:SetHasAlpha(true)
			group2:AddChild(color)
			color = UIF:ColorSelect("Frost Rune", db.RuneColors, RUNETYPE_FROST,  mRunes.ApplyAllSettings)
			color:SetRelativeWidth(0.20)
			color:SetHasAlpha(true)
			group2:AddChild(color)
			color = UIF:ColorSelect("Death Rune", db.RuneColors, RUNETYPE_DEATH,  mRunes.ApplyAllSettings)
			color:SetRelativeWidth(0.20)
			color:SetHasAlpha(true)
			group2:AddChild(color)			
			group:AddChild(group2)
			
			group2 = UIF:InlineGroup2("Size")
			slider = UIF:Slider("Width", db, "RuneWidth", 5, 500, 0.1, mRunes.ApplyAllSettings)
			group2:AddChild(slider)
			slider = UIF:Slider("Height", db, "RuneHeight", 5, 500, 0.1, mRunes.ApplyAllSettings)
			group2:AddChild(slider)
			slider = UIF:Slider("Spacing", db, "RuneSpacing", 0, 50, 0.1, mRunes.ApplyAllSettings)
			group2:AddChild(slider)			
			group:AddChild(group2)			
		end
	end

	function mRunes:SetupLDB()
		local LDB = LibStub("LibDataBroker-1.1")
		if not LDB then return end
		
		local l = LDB:NewDataObject("mRunes")
		l.type = "launcher"
		l.icon = [[Interface\Icons\Spell_DeathKnight_RuneTap]]
		l.tocname = "mRunes"
		l.label = "mRunes"
		l.OnClick = function(self, button)
			mRunes:ToggleConfig()
		end
		l.OnTooltipShow = function(tt)
			tt:AddLine("mRunes")
			tt:AddLine("Click to toggle configuration")
		end
		
		self.ldbojb = l
	end
end